#ifndef QPID_CLIENT_ASYNCSESSION_0_10_H
#define QPID_CLIENT_ASYNCSESSION_0_10_H
/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

///
/// This file was automatically generated from the AMQP specification.
/// Do not edit.
///


#include "qpid/client/no_keyword/AsyncSession_0_10.h"
#include "qpid/client/arg.h"

namespace qpid {
namespace client {

/**
 * AMQP 0-10 session API with keyword arguments.
 * This class provides the same set of functions as no_keyword::AsyncSession_0_10, but also
 * allows parameters be passed using keywords. The keyword is the
 * parameter name in the namespace "arg".
 * 
 * For example given the normal function "foo(int x=0, int y=0, int z=0)"
 * you could call it in either of the following ways:
 * 
 * @code
 * session.foo(1,2,3);             // Normal no keywords
 * session.foo(arg::z=3, arg::x=1); // Keywords and a default
 * @endcode
 * 
 * The keyword functions are easy to use but their declarations are hard
 * to read. You may find it easier to read the documentation for no_keyword::AsyncSession_0_10
 * which provides the same set of functions using normal non-keyword
 * declarations.
 * 
 * \ingroup clientapi
 * 
 */
class AsyncSession_0_10:
    public no_keyword::AsyncSession_0_10
{
  public:
    
    AsyncSession_0_10();
    AsyncSession_0_10(const SessionBase_0_10& other);
    AsyncSession_0_10& operator=(const SessionBase_0_10& other);
  private:
    typedef boost::parameter::parameters<arg::keyword_tags::commandId,arg::keyword_tags::value> ExecutionResultParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::errorCode,arg::keyword_tags::commandId,arg::keyword_tags::classCode,arg::keyword_tags::commandCode,arg::keyword_tags::fieldIndex,arg::keyword_tags::description,arg::keyword_tags::errorInfo> ExecutionExceptionParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination,arg::keyword_tags::acceptMode,arg::keyword_tags::acquireMode,arg::keyword_tags::content> MessageTransferParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::transfers> MessageAcceptParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::transfers,arg::keyword_tags::code,arg::keyword_tags::text> MessageRejectParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::transfers,arg::keyword_tags::setRedelivered> MessageReleaseParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::transfers> MessageAcquireParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination,arg::keyword_tags::resumeId> MessageResumeParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue,arg::keyword_tags::destination,arg::keyword_tags::acceptMode,arg::keyword_tags::acquireMode,arg::keyword_tags::exclusive,arg::keyword_tags::resumeId,arg::keyword_tags::resumeTtl,arg::keyword_tags::arguments> MessageSubscribeParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination> MessageCancelParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination,arg::keyword_tags::flowMode> MessageSetFlowModeParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination,arg::keyword_tags::unit,arg::keyword_tags::value> MessageFlowParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination> MessageFlushParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::destination> MessageStopParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid,arg::keyword_tags::join,arg::keyword_tags::resume> DtxStartParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid,arg::keyword_tags::fail,arg::keyword_tags::suspend> DtxEndParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid,arg::keyword_tags::onePhase> DtxCommitParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid> DtxForgetParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid> DtxGetTimeoutParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid> DtxPrepareParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid> DtxRollbackParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::xid,arg::keyword_tags::timeout> DtxSetTimeoutParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::exchange,arg::keyword_tags::type,arg::keyword_tags::alternateExchange,arg::keyword_tags::passive,arg::keyword_tags::durable,arg::keyword_tags::autoDelete,arg::keyword_tags::arguments> ExchangeDeclareParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::exchange,arg::keyword_tags::ifUnused> ExchangeDeleteParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::name> ExchangeQueryParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue,arg::keyword_tags::exchange,arg::keyword_tags::bindingKey,arg::keyword_tags::arguments> ExchangeBindParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue,arg::keyword_tags::exchange,arg::keyword_tags::bindingKey> ExchangeUnbindParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::exchange,arg::keyword_tags::queue,arg::keyword_tags::bindingKey,arg::keyword_tags::arguments> ExchangeBoundParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue,arg::keyword_tags::alternateExchange,arg::keyword_tags::passive,arg::keyword_tags::durable,arg::keyword_tags::exclusive,arg::keyword_tags::autoDelete,arg::keyword_tags::arguments> QueueDeclareParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue,arg::keyword_tags::ifUnused,arg::keyword_tags::ifEmpty> QueueDeleteParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue> QueuePurgeParameters;
    
    typedef boost::parameter::parameters<arg::keyword_tags::queue> QueueQueryParameters;
    
    friend class Connection;
  public:
    BOOST_PARAMETER_MEMFUN(Completion, executionResult, 0, 2, ExecutionResultParameters) {
        return no_keyword::AsyncSession_0_10::executionResult(
            p[arg::commandId|SequenceNumber()],
            p[arg::value|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, executionException, 0, 7, ExecutionExceptionParameters) {
        return no_keyword::AsyncSession_0_10::executionException(
            p[arg::errorCode|0],
            p[arg::commandId|SequenceNumber()],
            p[arg::classCode|0],
            p[arg::commandCode|0],
            p[arg::fieldIndex|0],
            p[arg::description|string()],
            p[arg::errorInfo|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageTransfer, 0, 4, MessageTransferParameters) {
        return no_keyword::AsyncSession_0_10::messageTransfer(
            p[arg::destination|string()],
            p[arg::acceptMode|0],
            p[arg::acquireMode|0],
            p[arg::content|DefaultContent(std::string())]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageAccept, 0, 1, MessageAcceptParameters) {
        return no_keyword::AsyncSession_0_10::messageAccept(
            p[arg::transfers|SequenceSet()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageReject, 0, 3, MessageRejectParameters) {
        return no_keyword::AsyncSession_0_10::messageReject(
            p[arg::transfers|SequenceSet()],
            p[arg::code|0],
            p[arg::text|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageRelease, 0, 2, MessageReleaseParameters) {
        return no_keyword::AsyncSession_0_10::messageRelease(
            p[arg::transfers|SequenceSet()],
            p[arg::setRedelivered|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::MessageAcquireResult>, messageAcquire, 0, 1, MessageAcquireParameters) {
        return no_keyword::AsyncSession_0_10::messageAcquire(
            p[arg::transfers|SequenceSet()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::MessageResumeResult>, messageResume, 0, 2, MessageResumeParameters) {
        return no_keyword::AsyncSession_0_10::messageResume(
            p[arg::destination|string()],
            p[arg::resumeId|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageSubscribe, 0, 8, MessageSubscribeParameters) {
        return no_keyword::AsyncSession_0_10::messageSubscribe(
            p[arg::queue|string()],
            p[arg::destination|string()],
            p[arg::acceptMode|0],
            p[arg::acquireMode|0],
            p[arg::exclusive|false],
            p[arg::resumeId|string()],
            p[arg::resumeTtl|0],
            p[arg::arguments|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageCancel, 0, 1, MessageCancelParameters) {
        return no_keyword::AsyncSession_0_10::messageCancel(
            p[arg::destination|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageSetFlowMode, 0, 2, MessageSetFlowModeParameters) {
        return no_keyword::AsyncSession_0_10::messageSetFlowMode(
            p[arg::destination|string()],
            p[arg::flowMode|0]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageFlow, 0, 3, MessageFlowParameters) {
        return no_keyword::AsyncSession_0_10::messageFlow(
            p[arg::destination|string()],
            p[arg::unit|0],
            p[arg::value|0]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageFlush, 0, 1, MessageFlushParameters) {
        return no_keyword::AsyncSession_0_10::messageFlush(
            p[arg::destination|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, messageStop, 0, 1, MessageStopParameters) {
        return no_keyword::AsyncSession_0_10::messageStop(
            p[arg::destination|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::XaResult>, dtxStart, 0, 3, DtxStartParameters) {
        return no_keyword::AsyncSession_0_10::dtxStart(
            p[arg::xid|Xid()],
            p[arg::join|false],
            p[arg::resume|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::XaResult>, dtxEnd, 0, 3, DtxEndParameters) {
        return no_keyword::AsyncSession_0_10::dtxEnd(
            p[arg::xid|Xid()],
            p[arg::fail|false],
            p[arg::suspend|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::XaResult>, dtxCommit, 0, 2, DtxCommitParameters) {
        return no_keyword::AsyncSession_0_10::dtxCommit(
            p[arg::xid|Xid()],
            p[arg::onePhase|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, dtxForget, 0, 1, DtxForgetParameters) {
        return no_keyword::AsyncSession_0_10::dtxForget(
            p[arg::xid|Xid()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::DtxGetTimeoutResult>, dtxGetTimeout, 0, 1, DtxGetTimeoutParameters) {
        return no_keyword::AsyncSession_0_10::dtxGetTimeout(
            p[arg::xid|Xid()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::XaResult>, dtxPrepare, 0, 1, DtxPrepareParameters) {
        return no_keyword::AsyncSession_0_10::dtxPrepare(
            p[arg::xid|Xid()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::XaResult>, dtxRollback, 0, 1, DtxRollbackParameters) {
        return no_keyword::AsyncSession_0_10::dtxRollback(
            p[arg::xid|Xid()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, dtxSetTimeout, 0, 2, DtxSetTimeoutParameters) {
        return no_keyword::AsyncSession_0_10::dtxSetTimeout(
            p[arg::xid|Xid()],
            p[arg::timeout|0]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, exchangeDeclare, 0, 7, ExchangeDeclareParameters) {
        return no_keyword::AsyncSession_0_10::exchangeDeclare(
            p[arg::exchange|string()],
            p[arg::type|string()],
            p[arg::alternateExchange|string()],
            p[arg::passive|false],
            p[arg::durable|false],
            p[arg::autoDelete|false],
            p[arg::arguments|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, exchangeDelete, 0, 2, ExchangeDeleteParameters) {
        return no_keyword::AsyncSession_0_10::exchangeDelete(
            p[arg::exchange|string()],
            p[arg::ifUnused|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::ExchangeQueryResult>, exchangeQuery, 0, 1, ExchangeQueryParameters) {
        return no_keyword::AsyncSession_0_10::exchangeQuery(
            p[arg::name|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, exchangeBind, 0, 4, ExchangeBindParameters) {
        return no_keyword::AsyncSession_0_10::exchangeBind(
            p[arg::queue|string()],
            p[arg::exchange|string()],
            p[arg::bindingKey|string()],
            p[arg::arguments|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, exchangeUnbind, 0, 3, ExchangeUnbindParameters) {
        return no_keyword::AsyncSession_0_10::exchangeUnbind(
            p[arg::queue|string()],
            p[arg::exchange|string()],
            p[arg::bindingKey|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::ExchangeBoundResult>, exchangeBound, 0, 4, ExchangeBoundParameters) {
        return no_keyword::AsyncSession_0_10::exchangeBound(
            p[arg::exchange|string()],
            p[arg::queue|string()],
            p[arg::bindingKey|string()],
            p[arg::arguments|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, queueDeclare, 0, 7, QueueDeclareParameters) {
        return no_keyword::AsyncSession_0_10::queueDeclare(
            p[arg::queue|string()],
            p[arg::alternateExchange|string()],
            p[arg::passive|false],
            p[arg::durable|false],
            p[arg::exclusive|false],
            p[arg::autoDelete|false],
            p[arg::arguments|FieldTable()]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, queueDelete, 0, 3, QueueDeleteParameters) {
        return no_keyword::AsyncSession_0_10::queueDelete(
            p[arg::queue|string()],
            p[arg::ifUnused|false],
            p[arg::ifEmpty|false]);
    }
    
    BOOST_PARAMETER_MEMFUN(Completion, queuePurge, 0, 1, QueuePurgeParameters) {
        return no_keyword::AsyncSession_0_10::queuePurge(
            p[arg::queue|string()]);
    }
    
    BOOST_PARAMETER_MEMFUN(TypedResult<qpid::framing::QueueQueryResult>, queueQuery, 0, 1, QueueQueryParameters) {
        return no_keyword::AsyncSession_0_10::queueQuery(
            p[arg::queue|string()]);
    }
    
};
/** Conversion to AsyncSession_0_10 from another session type */
inline AsyncSession_0_10 async(const SessionBase_0_10& other) { return (other); }

inline AsyncSession_0_10::AsyncSession_0_10() {}
inline AsyncSession_0_10::AsyncSession_0_10(const SessionBase_0_10& other) {
    *this = other;
}
inline AsyncSession_0_10& AsyncSession_0_10::operator=(const SessionBase_0_10& other) {
    impl = static_cast<const AsyncSession_0_10&>(other).impl;
    return *this;
}

}} // namespace qpid::client

#endif  /*!QPID_CLIENT_ASYNCSESSION_0_10_H*/
