
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Connection.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Connection::packageName  = string ("qpid");
string  Connection::className    = string ("connection");
uint8_t Connection::md5Sum[16]   =
    {0xb7,0x53,0x3b,0x53,0x90,0x1,0xab,0xba,0x8b,0xe5,0xc9,0xe3,0x10,0x85,0x95,0x6c};

Connection::Connection (Manageable* _core, Manageable* _parent, std::string _address, uint8_t _incoming) :
    ManagementObject(_core)
    ,address(_address),incoming(_incoming)
{
    vhostRef = _parent->GetManagementObject ()->getObjectId ();
    closing = 0;
    authIdentity = "";
    framesFromClient = 0;
    framesToClient = 0;
    bytesFromClient = 0;
    bytesToClient = 0;

}

Connection::~Connection () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Connection::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (3); // Config Element Count
    buf.putShort       (6); // Inst Element Count
    buf.putShort       (1); // Method Count
    buf.putShort       (0); // Event Count

    // Properties
    ft = FieldTable ();
    ft.setString (NAME,   "vhostRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "address");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "incoming");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  0);
    buf.put (ft);


    // Statistics
    ft = FieldTable ();
    ft.setString (NAME,   "closing");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setString (DESC,   "This client is closing by management request");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "authIdentity");
    ft.setInt    (TYPE,   TYPE_SSTR);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "framesFromClient");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "framesToClient");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bytesFromClient");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bytesToClient");
    ft.setInt    (TYPE,   TYPE_U64);
    buf.put (ft);


    // Methods
    ft = FieldTable ();
    ft.setString (NAME,     "close");
    ft.setInt    (ARGCOUNT, 0);
    buf.put (ft);


    // Events

}

void Connection::writeProperties (Buffer& buf)
{
    sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (vhostRef);
    buf.putShortString (address);
    buf.putOctet (incoming?1:0);

}

void Connection::writeStatistics (Buffer& buf, bool skipHeaders)
{
    sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;


    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putOctet (closing?1:0);
    buf.putShortString (authIdentity);
    buf.putLongLong (framesFromClient);
    buf.putLongLong (framesToClient);
    buf.putLongLong (bytesFromClient);
    buf.putLongLong (bytesToClient);


    // Maintenance of hi-lo statistics

}

void Connection::doMethod (string methodName, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    if (methodName == "close")
    {
        ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_CLOSE, ioArgs);
        outBuf.putLong        (status);
        outBuf.putShortString (Manageable::StatusText (status));
        return;
    }

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

