
#ifndef _MANAGEMENT_CONNECTION_
#define _MANAGEMENT_CONNECTION_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"
#include "qpid/sys/AtomicCount.h"

namespace qpid {
namespace management {

class Connection : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Properties
    uint64_t vhostRef;
    std::string address;
    uint8_t incoming;

    // Statistics
    uint8_t  closing;
    std::string  authIdentity;
    uint64_t  framesFromClient;
    uint64_t  framesToClient;
    uint64_t  bytesFromClient;
    uint64_t  bytesToClient;

    // Private Methods
    static void writeSchema (qpid::framing::Buffer& buf);
    void writeProperties    (qpid::framing::Buffer& buf);
    void writeStatistics    (qpid::framing::Buffer& buf,
                             bool skipHeaders = false);
    void doMethod           (std::string            methodName,
                             qpid::framing::Buffer& inBuf,
                             qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Connection> shared_ptr;

    Connection (Manageable* coreObject, Manageable* _parent, std::string _address, uint8_t _incoming);
    ~Connection (void);

    

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_CLOSE = 1;

    // Accessor Methods
    inline void set_closing (uint8_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        closing = val;
        instChanged = true;
    }
    inline void set_authIdentity (std::string val){
        sys::Mutex::ScopedLock mutex(accessLock);
        authIdentity = val;
        instChanged = true;
    }
    inline void inc_framesFromClient (uint64_t by = 1){
        if (by == 1)
            ++framesFromClient;
        else
            framesFromClient += by;
        instChanged = true;
    }
    inline void dec_framesFromClient (uint64_t by = 1){
        if (by == 1)
            framesFromClient--;
        else
            framesFromClient -= by;
        instChanged = true;
    }
    inline void set_framesFromClient (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        framesFromClient = val;
        instChanged = true;
    }
    inline void inc_framesToClient (uint64_t by = 1){
        if (by == 1)
            ++framesToClient;
        else
            framesToClient += by;
        instChanged = true;
    }
    inline void dec_framesToClient (uint64_t by = 1){
        if (by == 1)
            framesToClient--;
        else
            framesToClient -= by;
        instChanged = true;
    }
    inline void set_framesToClient (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        framesToClient = val;
        instChanged = true;
    }
    inline void inc_bytesFromClient (uint64_t by = 1){
        if (by == 1)
            ++bytesFromClient;
        else
            bytesFromClient += by;
        instChanged = true;
    }
    inline void dec_bytesFromClient (uint64_t by = 1){
        if (by == 1)
            bytesFromClient--;
        else
            bytesFromClient -= by;
        instChanged = true;
    }
    inline void set_bytesFromClient (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        bytesFromClient = val;
        instChanged = true;
    }
    inline void inc_bytesToClient (uint64_t by = 1){
        if (by == 1)
            ++bytesToClient;
        else
            bytesToClient += by;
        instChanged = true;
    }
    inline void dec_bytesToClient (uint64_t by = 1){
        if (by == 1)
            bytesToClient--;
        else
            bytesToClient -= by;
        instChanged = true;
    }
    inline void set_bytesToClient (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        bytesToClient = val;
        instChanged = true;
    }

};

}}

#endif  /*!_MANAGEMENT_CONNECTION_*/
