
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/log/Statement.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/management/Manageable.h" 
#include "Exchange.h"


using namespace qpid::management;
using namespace qpid::sys;
using namespace qpid::framing;
using           std::string;

string  Exchange::packageName  = string ("qpid");
string  Exchange::className    = string ("exchange");
uint8_t Exchange::md5Sum[16]   =
    {0x6d,0x2c,0x4b,0x49,0x6c,0x28,0x58,0x5c,0x1a,0x9e,0x5d,0x18,0x7a,0x7,0xc9,0xd4};

Exchange::Exchange (Manageable* _core, Manageable* _parent, std::string _name, uint8_t _durable) :
    ManagementObject(_core)
    ,name(_name),durable(_durable)
{
    vhostRef = _parent->GetManagementObject ()->getObjectId ();
    producerCount = 0;
    producerCountHigh = 0;
    producerCountLow  = 0;
    bindingCount = 0;
    bindingCountHigh = 0;
    bindingCountLow  = 0;
    msgReceives = 0;
    msgDrops = 0;
    msgRoutes = 0;
    byteReceives = 0;
    byteDrops = 0;
    byteRoutes = 0;

}

Exchange::~Exchange () {}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string INDEX("index");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Exchange::writeSchema (Buffer& buf)
{
    FieldTable ft;

    // Schema class header:
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (4); // Config Element Count
    buf.putShort       (12); // Inst Element Count
    buf.putShort       (0); // Method Count
    buf.putShort       (0); // Event Count

    // Properties
    ft = FieldTable ();
    ft.setString (NAME,   "vhostRef");
    ft.setInt    (TYPE,   TYPE_REF);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "name");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  1);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "type");
    ft.setInt    (TYPE,   TYPE_SSTR);
    ft.setInt    (ACCESS, ACCESS_RO);
    ft.setInt    (INDEX,  0);
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "durable");
    ft.setInt    (TYPE,   TYPE_BOOL);
    ft.setInt    (ACCESS, ACCESS_RC);
    ft.setInt    (INDEX,  0);
    buf.put (ft);


    // Statistics
    ft = FieldTable ();
    ft.setString (NAME,   "producerCount");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "producerCountHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange (High)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "producerCountLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current producers on exchange (Low)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindingCount");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindingCountHigh");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings (High)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "bindingCountLow");
    ft.setInt    (TYPE,   TYPE_U32);
    ft.setString (DESC,   "Current bindings (Low)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgReceives");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total messages received");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgDrops");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total messages dropped (no matching key)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "msgRoutes");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total routed messages");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteReceives");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total bytes received");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteDrops");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total bytes dropped (no matching key)");
    buf.put (ft);

    ft = FieldTable ();
    ft.setString (NAME,   "byteRoutes");
    ft.setInt    (TYPE,   TYPE_U64);
    ft.setString (DESC,   "Total routed bytes");
    buf.put (ft);


    // Methods

    // Events

}

void Exchange::writeProperties (Buffer& buf)
{
    sys::Mutex::ScopedLock mutex(accessLock);
    configChanged = false;

    writeTimestamps (buf);
    buf.putLongLong (vhostRef);
    buf.putShortString (name);
    buf.putShortString (type);
    buf.putOctet (durable?1:0);

}

void Exchange::writeStatistics (Buffer& buf, bool skipHeaders)
{
    sys::Mutex::ScopedLock mutex(accessLock);
    instChanged = false;


    if (!skipHeaders)
        writeTimestamps (buf);
    buf.putLong (producerCount);
    buf.putLong (producerCountHigh);
    buf.putLong (producerCountLow);
    buf.putLong (bindingCount);
    buf.putLong (bindingCountHigh);
    buf.putLong (bindingCountLow);
    buf.putLongLong (msgReceives);
    buf.putLongLong (msgDrops);
    buf.putLongLong (msgRoutes);
    buf.putLongLong (byteReceives);
    buf.putLongLong (byteDrops);
    buf.putLongLong (byteRoutes);


    // Maintenance of hi-lo statistics
    producerCountHigh = producerCount;
    producerCountLow  = producerCount;
    bindingCountHigh = bindingCount;
    bindingCountLow  = bindingCount;

}

void Exchange::doMethod (string, Buffer&, Buffer& outBuf)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;

    outBuf.putLong        (status);
    outBuf.putShortString (Manageable::StatusText (status));
}

