
#ifndef _MANAGEMENT_QUEUE_
#define _MANAGEMENT_QUEUE_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"
#include "qpid/sys/AtomicCount.h"

namespace qpid {
namespace management {

class Queue : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Properties
    uint64_t vhostRef;
    std::string name;
    uint8_t durable;
    uint8_t autoDelete;
    uint8_t exclusive;
    framing::FieldTable arguments;

    // Statistics
    uint64_t  msgTotalEnqueues;
    uint64_t  msgTotalDequeues;
    uint64_t  msgTxnEnqueues;
    uint64_t  msgTxnDequeues;
    uint64_t  msgPersistEnqueues;
    uint64_t  msgPersistDequeues;
    qpid::sys::AtomicCount  msgDepth;
    uint32_t  byteDepth;
    uint64_t  byteTotalEnqueues;
    uint64_t  byteTotalDequeues;
    uint64_t  byteTxnEnqueues;
    uint64_t  byteTxnDequeues;
    uint64_t  bytePersistEnqueues;
    uint64_t  bytePersistDequeues;
    uint64_t  enqueueTxnStarts;
    uint64_t  enqueueTxnCommits;
    uint64_t  enqueueTxnRejects;
    uint32_t  enqueueTxnCount;
    uint32_t  enqueueTxnCountHigh;
    uint32_t  enqueueTxnCountLow;
    uint64_t  dequeueTxnStarts;
    uint64_t  dequeueTxnCommits;
    uint64_t  dequeueTxnRejects;
    uint32_t  dequeueTxnCount;
    uint32_t  dequeueTxnCountHigh;
    uint32_t  dequeueTxnCountLow;
    uint32_t  consumerCount;
    uint32_t  consumerCountHigh;
    uint32_t  consumerCountLow;
    uint32_t  bindingCount;
    uint32_t  bindingCountHigh;
    uint32_t  bindingCountLow;
    uint32_t  unackedMessages;
    uint32_t  unackedMessagesHigh;
    uint32_t  unackedMessagesLow;
    uint64_t  messageLatencyCount;
    uint64_t  messageLatencyTotal;
    uint64_t  messageLatencyMin;
    uint64_t  messageLatencyMax;

    // Private Methods
    static void writeSchema (qpid::framing::Buffer& buf);
    void writeProperties    (qpid::framing::Buffer& buf);
    void writeStatistics    (qpid::framing::Buffer& buf,
                             bool skipHeaders = false);
    void doMethod           (std::string            methodName,
                             qpid::framing::Buffer& inBuf,
                             qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Queue> shared_ptr;

    Queue (Manageable* coreObject, Manageable* _parent, std::string _name, uint8_t _durable, uint8_t _autoDelete, uint8_t _exclusive);
    ~Queue (void);

    

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_PURGE = 1;

    // Accessor Methods
    inline void set_arguments (framing::FieldTable val){
        sys::Mutex::ScopedLock mutex(accessLock);
        arguments = val;
        configChanged = true;
    }
    inline void inc_msgTotalEnqueues (uint64_t by = 1){
        if (by == 1)
            ++msgTotalEnqueues;
        else
            msgTotalEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgTotalEnqueues (uint64_t by = 1){
        if (by == 1)
            msgTotalEnqueues--;
        else
            msgTotalEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgTotalEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgTotalEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgTotalDequeues (uint64_t by = 1){
        if (by == 1)
            ++msgTotalDequeues;
        else
            msgTotalDequeues += by;
        instChanged = true;
    }
    inline void dec_msgTotalDequeues (uint64_t by = 1){
        if (by == 1)
            msgTotalDequeues--;
        else
            msgTotalDequeues -= by;
        instChanged = true;
    }
    inline void set_msgTotalDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgTotalDequeues = val;
        instChanged = true;
    }
    inline void inc_msgTxnEnqueues (uint64_t by = 1){
        if (by == 1)
            ++msgTxnEnqueues;
        else
            msgTxnEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgTxnEnqueues (uint64_t by = 1){
        if (by == 1)
            msgTxnEnqueues--;
        else
            msgTxnEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgTxnEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgTxnEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgTxnDequeues (uint64_t by = 1){
        if (by == 1)
            ++msgTxnDequeues;
        else
            msgTxnDequeues += by;
        instChanged = true;
    }
    inline void dec_msgTxnDequeues (uint64_t by = 1){
        if (by == 1)
            msgTxnDequeues--;
        else
            msgTxnDequeues -= by;
        instChanged = true;
    }
    inline void set_msgTxnDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgTxnDequeues = val;
        instChanged = true;
    }
    inline void inc_msgPersistEnqueues (uint64_t by = 1){
        if (by == 1)
            ++msgPersistEnqueues;
        else
            msgPersistEnqueues += by;
        instChanged = true;
    }
    inline void dec_msgPersistEnqueues (uint64_t by = 1){
        if (by == 1)
            msgPersistEnqueues--;
        else
            msgPersistEnqueues -= by;
        instChanged = true;
    }
    inline void set_msgPersistEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgPersistEnqueues = val;
        instChanged = true;
    }
    inline void inc_msgPersistDequeues (uint64_t by = 1){
        if (by == 1)
            ++msgPersistDequeues;
        else
            msgPersistDequeues += by;
        instChanged = true;
    }
    inline void dec_msgPersistDequeues (uint64_t by = 1){
        if (by == 1)
            msgPersistDequeues--;
        else
            msgPersistDequeues -= by;
        instChanged = true;
    }
    inline void set_msgPersistDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        msgPersistDequeues = val;
        instChanged = true;
    }
    inline void inc_msgDepth (){
        ++msgDepth;
        instChanged = true;
    }
    inline void dec_msgDepth (){
        --msgDepth;
        instChanged = true;
    }
    inline void inc_byteDepth (uint32_t by = 1){
        if (by == 1)
            ++byteDepth;
        else
            byteDepth += by;
        instChanged = true;
    }
    inline void dec_byteDepth (uint32_t by = 1){
        if (by == 1)
            byteDepth--;
        else
            byteDepth -= by;
        instChanged = true;
    }
    inline void set_byteDepth (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        byteDepth = val;
        instChanged = true;
    }
    inline void inc_byteTotalEnqueues (uint64_t by = 1){
        if (by == 1)
            ++byteTotalEnqueues;
        else
            byteTotalEnqueues += by;
        instChanged = true;
    }
    inline void dec_byteTotalEnqueues (uint64_t by = 1){
        if (by == 1)
            byteTotalEnqueues--;
        else
            byteTotalEnqueues -= by;
        instChanged = true;
    }
    inline void set_byteTotalEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        byteTotalEnqueues = val;
        instChanged = true;
    }
    inline void inc_byteTotalDequeues (uint64_t by = 1){
        if (by == 1)
            ++byteTotalDequeues;
        else
            byteTotalDequeues += by;
        instChanged = true;
    }
    inline void dec_byteTotalDequeues (uint64_t by = 1){
        if (by == 1)
            byteTotalDequeues--;
        else
            byteTotalDequeues -= by;
        instChanged = true;
    }
    inline void set_byteTotalDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        byteTotalDequeues = val;
        instChanged = true;
    }
    inline void inc_byteTxnEnqueues (uint64_t by = 1){
        if (by == 1)
            ++byteTxnEnqueues;
        else
            byteTxnEnqueues += by;
        instChanged = true;
    }
    inline void dec_byteTxnEnqueues (uint64_t by = 1){
        if (by == 1)
            byteTxnEnqueues--;
        else
            byteTxnEnqueues -= by;
        instChanged = true;
    }
    inline void set_byteTxnEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        byteTxnEnqueues = val;
        instChanged = true;
    }
    inline void inc_byteTxnDequeues (uint64_t by = 1){
        if (by == 1)
            ++byteTxnDequeues;
        else
            byteTxnDequeues += by;
        instChanged = true;
    }
    inline void dec_byteTxnDequeues (uint64_t by = 1){
        if (by == 1)
            byteTxnDequeues--;
        else
            byteTxnDequeues -= by;
        instChanged = true;
    }
    inline void set_byteTxnDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        byteTxnDequeues = val;
        instChanged = true;
    }
    inline void inc_bytePersistEnqueues (uint64_t by = 1){
        if (by == 1)
            ++bytePersistEnqueues;
        else
            bytePersistEnqueues += by;
        instChanged = true;
    }
    inline void dec_bytePersistEnqueues (uint64_t by = 1){
        if (by == 1)
            bytePersistEnqueues--;
        else
            bytePersistEnqueues -= by;
        instChanged = true;
    }
    inline void set_bytePersistEnqueues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        bytePersistEnqueues = val;
        instChanged = true;
    }
    inline void inc_bytePersistDequeues (uint64_t by = 1){
        if (by == 1)
            ++bytePersistDequeues;
        else
            bytePersistDequeues += by;
        instChanged = true;
    }
    inline void dec_bytePersistDequeues (uint64_t by = 1){
        if (by == 1)
            bytePersistDequeues--;
        else
            bytePersistDequeues -= by;
        instChanged = true;
    }
    inline void set_bytePersistDequeues (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        bytePersistDequeues = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnStarts (uint64_t by = 1){
        if (by == 1)
            ++enqueueTxnStarts;
        else
            enqueueTxnStarts += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnStarts (uint64_t by = 1){
        if (by == 1)
            enqueueTxnStarts--;
        else
            enqueueTxnStarts -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnStarts (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        enqueueTxnStarts = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnCommits (uint64_t by = 1){
        if (by == 1)
            ++enqueueTxnCommits;
        else
            enqueueTxnCommits += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnCommits (uint64_t by = 1){
        if (by == 1)
            enqueueTxnCommits--;
        else
            enqueueTxnCommits -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnCommits (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        enqueueTxnCommits = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnRejects (uint64_t by = 1){
        if (by == 1)
            ++enqueueTxnRejects;
        else
            enqueueTxnRejects += by;
        instChanged = true;
    }
    inline void dec_enqueueTxnRejects (uint64_t by = 1){
        if (by == 1)
            enqueueTxnRejects--;
        else
            enqueueTxnRejects -= by;
        instChanged = true;
    }
    inline void set_enqueueTxnRejects (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        enqueueTxnRejects = val;
        instChanged = true;
    }
    inline void inc_enqueueTxnCount (uint32_t by = 1){
        if (by == 1)
            ++enqueueTxnCount;
        else
            enqueueTxnCount += by;
        if (enqueueTxnCountHigh < enqueueTxnCount)
            enqueueTxnCountHigh = enqueueTxnCount;
        instChanged = true;
    }
    inline void dec_enqueueTxnCount (uint32_t by = 1){
        if (by == 1)
            enqueueTxnCount--;
        else
            enqueueTxnCount -= by;
        if (enqueueTxnCountLow > enqueueTxnCount)
            enqueueTxnCountLow = enqueueTxnCount;
        instChanged = true;
    }
    inline void set_enqueueTxnCount (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        enqueueTxnCount = val;
        if (enqueueTxnCountLow  > val)
            enqueueTxnCountLow  = val;
        if (enqueueTxnCountHigh < val)
            enqueueTxnCountHigh = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnStarts (uint64_t by = 1){
        if (by == 1)
            ++dequeueTxnStarts;
        else
            dequeueTxnStarts += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnStarts (uint64_t by = 1){
        if (by == 1)
            dequeueTxnStarts--;
        else
            dequeueTxnStarts -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnStarts (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        dequeueTxnStarts = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnCommits (uint64_t by = 1){
        if (by == 1)
            ++dequeueTxnCommits;
        else
            dequeueTxnCommits += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnCommits (uint64_t by = 1){
        if (by == 1)
            dequeueTxnCommits--;
        else
            dequeueTxnCommits -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnCommits (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        dequeueTxnCommits = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnRejects (uint64_t by = 1){
        if (by == 1)
            ++dequeueTxnRejects;
        else
            dequeueTxnRejects += by;
        instChanged = true;
    }
    inline void dec_dequeueTxnRejects (uint64_t by = 1){
        if (by == 1)
            dequeueTxnRejects--;
        else
            dequeueTxnRejects -= by;
        instChanged = true;
    }
    inline void set_dequeueTxnRejects (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        dequeueTxnRejects = val;
        instChanged = true;
    }
    inline void inc_dequeueTxnCount (uint32_t by = 1){
        if (by == 1)
            ++dequeueTxnCount;
        else
            dequeueTxnCount += by;
        if (dequeueTxnCountHigh < dequeueTxnCount)
            dequeueTxnCountHigh = dequeueTxnCount;
        instChanged = true;
    }
    inline void dec_dequeueTxnCount (uint32_t by = 1){
        if (by == 1)
            dequeueTxnCount--;
        else
            dequeueTxnCount -= by;
        if (dequeueTxnCountLow > dequeueTxnCount)
            dequeueTxnCountLow = dequeueTxnCount;
        instChanged = true;
    }
    inline void set_dequeueTxnCount (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        dequeueTxnCount = val;
        if (dequeueTxnCountLow  > val)
            dequeueTxnCountLow  = val;
        if (dequeueTxnCountHigh < val)
            dequeueTxnCountHigh = val;
        instChanged = true;
    }
    inline void inc_consumerCount (uint32_t by = 1){
        if (by == 1)
            ++consumerCount;
        else
            consumerCount += by;
        if (consumerCountHigh < consumerCount)
            consumerCountHigh = consumerCount;
        instChanged = true;
    }
    inline void dec_consumerCount (uint32_t by = 1){
        if (by == 1)
            consumerCount--;
        else
            consumerCount -= by;
        if (consumerCountLow > consumerCount)
            consumerCountLow = consumerCount;
        instChanged = true;
    }
    inline void set_consumerCount (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        consumerCount = val;
        if (consumerCountLow  > val)
            consumerCountLow  = val;
        if (consumerCountHigh < val)
            consumerCountHigh = val;
        instChanged = true;
    }
    inline void inc_bindingCount (uint32_t by = 1){
        if (by == 1)
            ++bindingCount;
        else
            bindingCount += by;
        if (bindingCountHigh < bindingCount)
            bindingCountHigh = bindingCount;
        instChanged = true;
    }
    inline void dec_bindingCount (uint32_t by = 1){
        if (by == 1)
            bindingCount--;
        else
            bindingCount -= by;
        if (bindingCountLow > bindingCount)
            bindingCountLow = bindingCount;
        instChanged = true;
    }
    inline void set_bindingCount (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        bindingCount = val;
        if (bindingCountLow  > val)
            bindingCountLow  = val;
        if (bindingCountHigh < val)
            bindingCountHigh = val;
        instChanged = true;
    }
    inline void inc_unackedMessages (uint32_t by = 1){
        if (by == 1)
            ++unackedMessages;
        else
            unackedMessages += by;
        if (unackedMessagesHigh < unackedMessages)
            unackedMessagesHigh = unackedMessages;
        instChanged = true;
    }
    inline void dec_unackedMessages (uint32_t by = 1){
        if (by == 1)
            unackedMessages--;
        else
            unackedMessages -= by;
        if (unackedMessagesLow > unackedMessages)
            unackedMessagesLow = unackedMessages;
        instChanged = true;
    }
    inline void set_unackedMessages (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        unackedMessages = val;
        if (unackedMessagesLow  > val)
            unackedMessagesLow  = val;
        if (unackedMessagesHigh < val)
            unackedMessagesHigh = val;
        instChanged = true;
    }
    inline void set_messageLatency (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        messageLatencyCount++;
        messageLatencyTotal += val;
        if (messageLatencyMin > val)
            messageLatencyMin = val;
        if (messageLatencyMax < val)
            messageLatencyMax = val;
        instChanged = true;
    }

};

}}

#endif  /*!_MANAGEMENT_QUEUE_*/
