
#ifndef _MANAGEMENT_SESSION_
#define _MANAGEMENT_SESSION_

//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/ManagementObject.h"
#include "qpid/framing/FieldTable.h"
#include "qpid/framing/Uuid.h"
#include "qpid/sys/AtomicCount.h"

namespace qpid {
namespace management {

class Session : public ManagementObject
{
  private:

    static std::string packageName;
    static std::string className;
    static uint8_t     md5Sum[16];

    // Properties
    uint64_t vhostRef;
    std::string name;
    uint16_t channelId;
    uint64_t connectionRef;
    uint32_t detachedLifespan;

    // Statistics
    uint8_t  attached;
    uint64_t  expireTime;
    uint32_t  framesOutstanding;

    // Private Methods
    static void writeSchema (qpid::framing::Buffer& buf);
    void writeProperties    (qpid::framing::Buffer& buf);
    void writeStatistics    (qpid::framing::Buffer& buf,
                             bool skipHeaders = false);
    void doMethod           (std::string            methodName,
                             qpid::framing::Buffer& inBuf,
                             qpid::framing::Buffer& outBuf);
    writeSchemaCall_t getWriteSchemaCall (void) { return writeSchema; }


  public:

    friend class PackageQpid;
    typedef boost::shared_ptr<Session> shared_ptr;

    Session (Manageable* coreObject, Manageable* _parent, std::string _name);
    ~Session (void);

    

    std::string getPackageName (void) { return packageName; }
    std::string getClassName   (void) { return className; }
    uint8_t*    getMd5Sum      (void) { return md5Sum; }

    // Method IDs
    static const uint32_t METHOD_SOLICITACK = 1;
    static const uint32_t METHOD_DETACH = 2;
    static const uint32_t METHOD_RESETLIFESPAN = 3;
    static const uint32_t METHOD_CLOSE = 4;

    // Accessor Methods
    inline void set_channelId (uint16_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        channelId = val;
        configChanged = true;
    }
    inline void set_connectionRef (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        connectionRef = val;
        configChanged = true;
    }
    inline void set_detachedLifespan (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        detachedLifespan = val;
        configChanged = true;
    }
    inline void set_attached (uint8_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        attached = val;
        instChanged = true;
    }
    inline void set_expireTime (uint64_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        expireTime = val;
        instChanged = true;
    }
    inline void inc_framesOutstanding (uint32_t by = 1){
        if (by == 1)
            ++framesOutstanding;
        else
            framesOutstanding += by;
        instChanged = true;
    }
    inline void dec_framesOutstanding (uint32_t by = 1){
        if (by == 1)
            framesOutstanding--;
        else
            framesOutstanding -= by;
        instChanged = true;
    }
    inline void set_framesOutstanding (uint32_t val){
        sys::Mutex::ScopedLock mutex(accessLock);
        framesOutstanding = val;
        instChanged = true;
    }

};

}}

#endif  /*!_MANAGEMENT_SESSION_*/
